@@ -333,10 +333,10 @@ Ready-made functions include
 `upcase-initials'.")
 
 (defvar mm-path-name-rewrite-functions nil
-  "*List of functions used for rewriting path names of MIME parts.
-This is used when viewing parts externally , and is meant for
-transforming the path name so that non-compliant programs can
-find the file where it's saved.
+  "*List of functions for rewriting the full file names of MIME parts.
+This is used when viewing parts externally, and is meant for
+transforming the absolute name so that non-compliant programs can find
+the file where it's saved.
 
 Each function takes a file name as input and returns a file name.")
 
@@ -1262,7 +1262,7 @@ If RECURSIVE, search recursively."
 		    (if notp
 			(not (equal (car ctl) type))
 		      (equal (car ctl) type)))
-	      (setq result (buffer-substring (point-min) (point-max)))))))
+	      (setq result (buffer-string))))))
       (forward-line 1)
       (setq start (point)))
     (when (and (not result) start)
@@ -1275,7 +1275,7 @@ If RECURSIVE, search recursively."
 		  (if notp
 		      (not (equal (car ctl) type))
 		    (equal (car ctl) type)))
-	    (setq result (buffer-substring (point-min) (point-max)))))))
+	    (setq result (buffer-string))))))
     result))
 
 (defvar mm-security-handle nil)
